@@ -10,12 +10,13 @@
 #undef HAVE_PROGRAM_INVOCATION_SHORT_NAME
 #undef HAVE_SIGACTION_H
 #undef HAVE_SIGSET_H
+#undef HAVE_GTKHTML
+#undef HAVE_GNOME_PRINT
 #undef BALSA_MAJOR
 #undef BALSA_PATCHLEVEL
 #undef BALSA_REVISION
 #undef BALSA_VERSION
 #undef BALSA_SHOW_ALL
-#undef BALSA_SHOW_INFO
 #undef PACKAGE
 #undef VERSION
 
@@ -29,13 +30,15 @@
 
 /* Shall we multithread? */
 #undef BALSA_USE_THREADS
-/* Try new init druid based on GOB */
-#undef BALSA_USE_EXPERIMENTAL_INIT
-/* Use GOB */
-#undef BALSA_HAVE_GOB
-/* We have root privs */
-#undef BALSA_SYSTEM_INSTALL
-/* We don't have root or won't use it */
-#undef BALSA_LOCAL_INSTALL
-/* Our data directory prefix */
-#undef BALSA_RESOURCE_PREFIX
+
+/* LDAP support? */
+#undef ENABLE_LDAP
+
+#undef BALSA_STD_PREFIX
+#undef BALSA_DATA_PREFIX
+#undef GNOME_STD_PREFIX
+#undef GNOME_LIB_PREFIX
+#undef GNOME_DATA_PREFIX
+
+/* The prefixes to look for balsa data, at least most likely */
+#undef BALSA_COMMON_PREFIXES
