@@ -157,9 +157,64 @@ void mutt_update_mailbox (BUFFY * b)
 }
 #endif
 
+/* buffy_add_mailbox:
+   adds given mailbox to the mailboxes to be checked periodically,
+   user and passwd are relevant only for IMAP mailboxes.
+   IMAP path is of standard form '{server:port}path'
+*/
+BUFFY *
+buffy_add_mailbox(const char *path, const char *user, const char *passwd)
+{
+    BUFFY **tmp;
+    /* simple check to avoid duplicates */
+    for (tmp = &Incoming; *tmp; tmp = &((*tmp)->next))
+    {
+      if (mutt_strcmp (path, (*tmp)->path) == 0)
+	break;
+    }
+
+    if (!*tmp)
+    {
+      *tmp = (BUFFY *) safe_calloc (1, sizeof (BUFFY));
+      (*tmp)->path   = safe_strdup (path);
+      (*tmp)->user   = user   ? safe_strdup (user)   : NULL;
+      (*tmp)->passwd = passwd ? safe_strdup (passwd) : NULL;
+      (*tmp)->next = NULL;
+    }
+
+    (*tmp)->new = 0;
+    (*tmp)->notified = 1;
+    (*tmp)->newly_created = 0;
+
+    return *tmp;
+}
+
+static BUFFY*
+buffy_mailbox_remove(BUFFY * bf)
+{
+    BUFFY ** tmp = &Incoming;
+
+    if(!*tmp) return NULL; /* strange error */
+    if(*tmp == bf)
+	*tmp = (*tmp)->next;
+    else {
+	while(*tmp && (*tmp)->next != bf) 
+	    tmp = &(*tmp)->next;
+	if( !*tmp ) return NULL; /* not found again, critical error! */
+	(*tmp)->next = bf->next;
+    }
+
+    fprintf(stderr,"Removing %s from buffy list\n", bf->path);
+    safe_free((void **) &bf->user);
+    safe_free((void **) &bf->passwd);
+    safe_free((void **) &bf->path);
+    safe_free((void **) &bf);
+    return (*tmp)->next;
+}
+
 int mutt_parse_mailboxes (BUFFER *path, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  BUFFY **tmp;
+  BUFFY *tmp;
   char buf[_POSIX_PATH_MAX];
 #ifdef BUFFY_SIZE
   struct stat sb;
@@ -174,36 +229,20 @@ int mutt_parse_mailboxes (BUFFER *path, BUFFER *s, unsigned long data, BUFFER *e
     /* Skip empty tokens. */
     if(!*buf) continue;
 
-    /* simple check to avoid duplicates */
-    for (tmp = &Incoming; *tmp; tmp = &((*tmp)->next))
-    {
-      if (mutt_strcmp (buf, (*tmp)->path) == 0)
-	break;
-    }
-
-    if (!*tmp)
-    {
-      *tmp = (BUFFY *) safe_calloc (1, sizeof (BUFFY));
-      (*tmp)->path = safe_strdup (buf);
-      (*tmp)->next = NULL;
-    }
-
-    (*tmp)->new = 0;
-    (*tmp)->notified = 1;
-    (*tmp)->newly_created = 0;
+    tmp = buffy_add_mailbox(buf, NULL, NULL);
 
 #ifdef BUFFY_SIZE
     /* for buffy_size, it is important that if the folder is new (tested by
      * reading it), the size is set to 0 so that later when we check we see
      * that it increased .  without buffy_size we probably don't care.
      */
-    if (stat ((*tmp)->path, &sb) == 0 && !test_new_folder ((*tmp)->path))
+    if (stat (tmp->path, &sb) == 0 && !test_new_folder (tmp->path))
     {
       /* some systems out there don't have an off_t type */
-      (*tmp)->size = (long) sb.st_size;
+      tmp->size = (long) sb.st_size;
     }
     else
-      (*tmp)->size = 0;
+      tmp->size = 0;
 #endif /* BUFFY_SIZE */
   }
   return 0;
@@ -231,10 +270,10 @@ int mutt_buffy_check (int force)
   static time_t last_imap_check = 0;
   int do_imap_check = 1;
 
-  if (ImapCheckTime)
+  if (ImapCheckTimeout)
   {
     time_t now = time (NULL);
-    if (!force && (now - last_imap_check < ImapCheckTime))
+    if (!force && (now - last_imap_check < ImapCheckTimeout))
       do_imap_check = 0;
     else
       last_imap_check = now;
@@ -348,12 +387,21 @@ int mutt_buffy_check (int force)
         /* poll on do_imap_check, else return cached value */
         if (do_imap_check)
         {
+	  char * old_user = ImapUser, *old_passwd = ImapPass;
+	  int res;
           tmp->new = 0;
-          if (imap_buffy_check (tmp->path) > 0)
+	  
+	  ImapUser = tmp->user;
+	  ImapPass = tmp->passwd;
+          if ( (res=imap_mailbox_check (tmp->path, 1))>0 )
           {
             BuffyCount++;
             tmp->new = 1;
-          }
+          } else if(res<0)
+	      /* failed (authorization?), remove it from list not
+		 to try the same wrong password over and over again */
+	      tmp = buffy_mailbox_remove(tmp);
+	  ImapUser = old_user, ImapPass = old_passwd;
         }
         else
           if (tmp->new)
@@ -367,6 +415,8 @@ int mutt_buffy_check (int force)
     else if (Context && Context->path)
       tmp->size = (long) sb.st_size;	/* update the size */
 #endif
+    /* tmp could be set to NULL by buffy_mailbox_remove */
+    if(!tmp) break;
 
     if (!tmp->new)
       tmp->notified = 0;
