@@ -23,7 +23,7 @@
 
 #include <stdio.h>
 #ifdef HAVE_UNISTD_H
-# include <unistd.h> /* needed for SEEK_SET */
+#include <unistd.h> /* needed for SEEK_SET */
 #endif
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -148,6 +148,14 @@ typedef enum
 #define M_THREAD_UNREAD		(1<<3)
 #define M_THREAD_NEXT_UNREAD	(1<<4)
 
+
+#ifdef M_READ
+/* FIXME: On Solaris (and probably others), M_READ is defined by the
+   streams headers (<sys/streams.h>).  We need to use a better
+   prefix.  */
+#undef M_READ
+#endif
+
 enum
 {
   /* modes for mutt_view_attachment() */
@@ -211,6 +219,9 @@ enum
 
   /* options for socket code */
   M_NEW_SOCKET,
+#ifdef USE_SSL
+  M_NEW_SSL_SOCKET,
+#endif
 
   /* Options for mutt_save_attachment */
   M_SAVE_APPEND,
@@ -277,13 +288,16 @@ enum
   OPTCOLLAPSEUNREAD,
   OPTCONFIRMAPPEND,
   OPTCONFIRMCREATE,
+  OPTDELETEUNTAG,
   OPTEDITHDRS,
+  OPTENCODEFROM,
+  OPTENVFROM,
   OPTFASTREPLY,
   OPTFCCATTACH,
+  OPTFCCCLEAR,
   OPTFOLLOWUPTO,
   OPTFORCENAME,
   OPTFORWDECODE,
-  OPTFORWWEEDHEADER,
   OPTFORWQUOTE,
   OPTHDRS,
   OPTHEADER,
@@ -291,7 +305,18 @@ enum
   OPTHIDDENHOST,
   OPTIGNORELISTREPLYTO,
 #ifdef USE_IMAP
-    OPTIMAPPASSIVE,
+  OPTIMAPLSUB,
+  OPTIMAPPASSIVE,
+  OPTIMAPSERVERNOISE,
+# ifdef USE_SSL
+  OPTIMAPFORCESSL,
+# endif
+#endif
+#ifdef USE_SSL
+  OPTSSLV2,
+  OPTSSLV3,
+  OPTTLSV1,
+  OPTSSLSYSTEMCERTS,
 #endif
   OPTIMPLICITAUTOVIEW,
   OPTMAILCAPSANITIZE,
@@ -305,17 +330,22 @@ enum
   OPTPAGERSTOP,
   OPTPIPEDECODE,
   OPTPIPESPLIT,
+  OPTPOPAPOP,
   OPTPOPDELETE,
   OPTPOPLAST,
+  OPTPRINTDECODE,
   OPTPROMPTAFTER,
   OPTREADONLY,
   OPTREPLYSELF,
   OPTRESOLVE,
   OPTREVALIAS,
   OPTREVNAME,
+  OPTREVREAL,
+  OPTRFC2047PARAMS,
   OPTSAVEADDRESS,
   OPTSAVEEMPTY,
   OPTSAVENAME,
+  OPTSCORE,
   OPTSIGDASHES,
   OPTSORTRE,
   OPTSTATUSONTOP,
@@ -332,18 +362,24 @@ enum
   OPTWRAP,
   OPTWRAPSEARCH,
   OPTWRITEBCC,		/* write out a bcc header? */
+  OPTXMAILER,
 
   /* PGP options */
   
-#ifdef _PGPPATH
+#ifdef HAVE_PGP
   OPTPGPAUTOSIGN,
   OPTPGPAUTOENCRYPT,
   OPTPGPLONGIDS,
   OPTPGPREPLYENCRYPT,
   OPTPGPREPLYSIGN,
+  OPTPGPREPLYSIGNENCRYPTED,
+#if 0
   OPTPGPENCRYPTSELF,
+#endif
+  OPTPGPRETAINABLESIG,
   OPTPGPSTRICTENC,
   OPTFORWDECRYPT,
+  OPTPGPSHOWUNUSABLE,
 #endif
 
   /* pseudo options */
@@ -567,6 +603,9 @@ typedef struct header
   struct header *last_sort; /* last message in subthread, for secondary SORT_LAST */
   char *tree;            /* character string to print thread tree */
 
+#ifdef USE_IMAP
+  void *data;            /* driver-specific data (only used by IMAP */
+#endif
 } HEADER;
 
 #include "mutt_regex.h"
