@@ -24,10 +24,7 @@
 #include "mutt.h"
 #include "mailbox.h"
 #include "mx.h"
-
-#ifdef BALSA_USE_THREADS
-#include "thread_msgs.h"
-#endif
+#include "md5.h"
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -37,6 +34,14 @@
 #include <string.h>
 #include <unistd.h>
 
+#ifdef LIBMUTT
+#define _(a) (a)
+#define sleep(a) 
+#endif
+
+/* XXXXX */
+/* #define BALSA_TEST_POP3 1 */
+
 static int getLine (int fd, char *s, int len)
 {
   char ch;
@@ -49,12 +54,18 @@ static int getLine (int fd, char *s, int len)
     if (ch == '\n')
     {
       *s = 0;
+      #ifdef BALSA_TEST_POP3
+      fprintf( stderr, "POP3 L: \"%s\"\n", (char *) (s - bytes) );
+      #endif
       return (bytes);
     }
     /* make sure not to overwrite the buffer */
     if (bytes == len - 1)
     {
       *s = 0;
+      #ifdef BALSA_TEST_POP3
+      fprintf( stderr, "POP3 L: \"%s\"\n", (char *) (s - bytes) );
+      #endif
       return bytes;
     }
   }
@@ -77,7 +88,84 @@ static int getPass (void)
   return 1;
 }
 #endif
-void mutt_fetchPopMail (void)
+
+#ifdef BALSA_TEST_POP3
+#define G_STMT_START ({
+#define G_STMT_END })
+#define mutt_error( str, args... ) G_STMT_START { fprintf( stderr, "POP3 E: " ); fprintf( stderr, str, ##args ); fprintf( stderr, "\n" ); mutt_error( str, ##args ); } G_STMT_END
+#define mutt_message( str, args... ) G_STMT_START { fprintf( stderr, "POP3 M: " ); fprintf( stderr, str, ##args ); fprintf( stderr, "\n" ); mutt_message( str, ##args ); } G_STMT_END
+#define mutt_perror( str ) G_STMT_START { fprintf( stderr, "POP3 P: %s\n", (str) ); mutt_perror( str ); } G_STMT_END
+#define write( sock, buffer, size ) G_STMT_START { fprintf( stderr, "POP3 O: \"%s\"\n", buffer ); write( sock, buffer, (size) ); } G_STMT_END
+#define DM( fmt, args... ) G_STMT_START { fprintf( stderr, "POP3 D: " ); fprintf( stderr, fmt, ##args ); fprintf( stderr, "\n" ); } G_STMT_END
+#else
+#define DM( fmt, args... ) 
+#endif
+
+#ifndef LIBMUTT
+static int getApopSecret (void) {
+  if (!PopApopPass || (PopApopPass[0] == '\0')) {
+    char tmp[SHORT_STRING];
+    memset(tmp, '\0', sizeof (tmp));
+    if (mutt_get_password ("APOP Shared Secret: ", tmp, sizeof (tmp)) != 0)
+      return 0;
+
+    PopApopPass = safe_strdup (tmp);
+  }
+  return 1;
+}
+#endif
+
+/* Get the Server Timestamp for APOP authentication -kabir */
+
+static int getApopStamp (char *buff, char *stamp) {
+  char *start;
+  char *finish;
+  size_t len;
+
+  start = strchr(buff, '<');
+  finish = strchr(buff, '>');
+
+  if( buff && stamp && start && finish ) {
+	len = strlen( start ) - strlen( finish ) + 1;
+
+	strncpy( stamp, start, len );
+
+	return 1;
+  }
+  else {
+	mutt_message (_("Unable to get APOP seed from server"));
+	return 0;
+  }
+}
+
+
+/* Compute the authentication hash to send to the server - kabir */
+
+static void computeAuthHash( char *stamp, char *hash ) {
+  MD5_CTX mdContext;
+  register unsigned char *dp;
+  register char *cp;
+  unsigned char *ep;
+  unsigned char digest[16];
+
+  MD5Init( &mdContext );
+  MD5Update( &mdContext, (unsigned char *)stamp, strlen( stamp ) );
+  /* BALSA: we just use PopPass, not PopAPass - the 2 fields are redundant*/
+  MD5Update( &mdContext, (unsigned char *)PopPass, strlen( PopPass ) );
+  MD5Final( digest, &mdContext );
+ 
+  cp = hash;
+  dp = digest;
+  for(ep = (dp = digest) + sizeof digest / sizeof digest[0];dp < ep;cp += 2) {
+    (void) sprintf (cp, "%02x", *dp);
+    dp++;
+  }
+
+  *cp = '\0';
+}
+
+
+void mutt_fetchPopMail (muttProgressCallback prog_cb)
 {
   struct sockaddr_in sin;
 #if SIZEOF_LONG == 4
@@ -88,8 +176,9 @@ void mutt_fetchPopMail (void)
   struct hostent *he;
   char buffer[2048];
   char msgbuf[SHORT_STRING];
+  char stamp[2048];
+  char authHash[BUFSIZ];
 #ifdef BALSA_USE_THREADS
-  MailThreadMessage *threadmsg;
   char threadbuf[160];
 #endif
   char uid[80], last_uid[80];
@@ -100,6 +189,8 @@ void mutt_fetchPopMail (void)
   CONTEXT ctx;
   MESSAGE *msg = NULL;
 
+  DM( "CheckPopMail head" );
+
   if (!PopHost)
   {
     mutt_error ("POP host is not defined.");
@@ -111,9 +202,27 @@ void mutt_fetchPopMail (void)
     mutt_error ("No POP username is defined.");
     return;
   }
+
+  DM( "Open buffer mailbox" );
+
+  if (mx_open_mailbox (NONULL(Spoolfile), M_APPEND, &ctx) == NULL) {
+	  mutt_error( "Cannot open the temporary mailbox to spool POP3 messages: \"%s\"", Spoolfile );
+	  return;
+  }
+
+
 #ifndef LIBMUTT   
-  if (!getPass ()) return;
+  /* handle apop secret fetching if APOP is turned on */
+  if( !option(OPTPOPAPOP) ) {
+      if (!getPass ()) return;
+  }
+  else {
+      if (!getApopSecret()) return;
+  }
 #endif
+
+  DM( "Begin connection" );
+
   s = socket (AF_INET, SOCK_STREAM, IPPROTO_IP);
 
   memset ((char *) &sin, 0, sizeof(sin));
@@ -126,6 +235,7 @@ void mutt_fetchPopMail (void)
     if ((he = gethostbyname (NONULL(PopHost))) == NULL)
     {
       mutt_error ("Could not find address for host %s.", PopHost);
+      mx_fastclose_mailbox (&ctx);
       return;
     }
     memcpy ((void *)&sin.sin_addr, *(he->h_addr_list), he->h_length);
@@ -138,11 +248,19 @@ void mutt_fetchPopMail (void)
   if (connect (s, (struct sockaddr *) &sin, sizeof (struct sockaddr_in)) == -1)
   {
     mutt_perror ("connect");
+
+    /* BALSA: This is needed to prevent deadlocks when failing */
+    /* to connect to a POP server */
+    mx_fastclose_mailbox (&ctx);
     return;
   }
-  
-  if (getLine (s, buffer, sizeof (buffer)) == -1)
-    goto fail;
+
+  DM( "Connected; hello" );
+
+  if (getLine (s, buffer, sizeof (buffer)) == -1) {
+	  mx_fastclose_mailbox (&ctx);
+	  goto fail;
+  }
 
   if (mutt_strncmp (buffer, "+OK", 3) != 0)
   {
@@ -151,24 +269,49 @@ void mutt_fetchPopMail (void)
     goto finish;
   }
 
-  snprintf (buffer, sizeof(buffer), "user %s\r\n", PopUser);
-  write (s, buffer, mutt_strlen (buffer));
-
-  if (getLine (s, buffer, sizeof (buffer)) == -1)
+  /* handle apop secret transmission, if needed -kabir */
+  if( option( OPTPOPAPOP ) ) {
+    
+    memset( stamp, '\0', sizeof( stamp ) );
+    if( !getApopStamp( buffer, stamp ) ) {
+      /* BALSA */ mx_fastclose_mailbox (&ctx);
+      goto fail;
+    }
+    
+    computeAuthHash( stamp, authHash );
+    
+    snprintf (buffer, sizeof(buffer), "apop %s %s\r\n", PopUser, authHash);
+    write (s, buffer, mutt_strlen (buffer));
+  } 
+  else {
+    DM( "hellod; user" );
+    
+    snprintf (buffer, sizeof(buffer), "user %s\r\n", PopUser);
+    write (s, buffer, mutt_strlen (buffer));
+    
+    if (getLine (s, buffer, sizeof (buffer)) == -1) {
+      mx_fastclose_mailbox (&ctx);
+      goto fail;
+    }
+    
+    if (mutt_strncmp (buffer, "+OK", 3) != 0)
+      {
+	mutt_remove_trailing_ws (buffer);
+	mutt_error (buffer);
+	goto finish;
+      }
+    
+    DM( "usered; password" );
+    
+    snprintf (buffer, sizeof(buffer), "pass %s\r\n", NONULL(PopPass));
+    write (s, buffer, mutt_strlen (buffer));
+  } 
+  
+  if (getLine (s, buffer, sizeof (buffer)) == -1) {
+    mx_fastclose_mailbox (&ctx);
     goto fail;
-
-  if (mutt_strncmp (buffer, "+OK", 3) != 0)
-  {
-    mutt_remove_trailing_ws (buffer);
-    mutt_error (buffer);
-    goto finish;
   }
   
-  snprintf (buffer, sizeof(buffer), "pass %s\r\n", NONULL(PopPass));
-  write (s, buffer, mutt_strlen (buffer));
-  
-  if (getLine (s, buffer, sizeof (buffer)) == -1)
-    goto fail;
 
   if (mutt_strncmp (buffer, "+OK", 3) != 0)
   {
@@ -180,12 +323,15 @@ void mutt_fetchPopMail (void)
     mutt_error (buffer[0] ? buffer : "Server closed connection!");
     goto finish;
   }
+  DM( "passworded; stat" );
   
   /* find out how many messages are in the mailbox. */
   write (s, "stat\r\n", 6);
   
-  if (getLine (s, buffer, sizeof (buffer)) == -1)
-    goto fail;
+  if (getLine (s, buffer, sizeof (buffer)) == -1) {
+	  mx_fastclose_mailbox (&ctx);
+	  goto fail;
+  }
 
   if (mutt_strncmp (buffer, "+OK", 3) != 0)
   {
@@ -196,15 +342,14 @@ void mutt_fetchPopMail (void)
   
   sscanf (buffer, "+OK %d %d", &msgs, &bytes);
 
+  DM( "Stat retrieved" );
+
   if (msgs == 0)
   {
     mutt_message ("No new mail in POP mailbox.");
     goto finish;
   }
 
-  if (mx_open_mailbox (NONULL(Spoolfile), M_APPEND, &ctx) == NULL)
-    goto finish;
-
  first_msg = 1;
 
 
@@ -216,6 +361,7 @@ void mutt_fetchPopMail (void)
  
  if ( !option (OPTPOPDELETE) )
  {
+	 DM( "Checking for seen UIDLs" );
 
   for( i = msgs; i > 0 ; i--) 
   {
@@ -226,6 +372,17 @@ void mutt_fetchPopMail (void)
   	
   	if (strncmp (buffer, "+OK", 3) != 0)
   	{
+	        /* BALSA: begin LAST support section */
+	        /* Check if "last" cmd is supported */
+	        write (s, "last\r\n", 6);
+		getLine (s, buffer, sizeof (buffer));
+		if (strncmp (buffer, "+OK", 3) == 0)
+		{       
+		    sscanf (buffer, "+OK %d", &first_msg);
+		    first_msg++; /* fix the off-by-one msg index  */
+		    break;
+		} 
+	        /* BALSA: end LAST support section */
   		mutt_remove_trailing_ws( buffer);
   		mutt_error (buffer);
   		goto finish; 
@@ -235,7 +392,7 @@ void mutt_fetchPopMail (void)
 
 	if( i == msgs )  
 	{
-	   strcpy( last_uid, uid ); // save uid of the last message for exit
+	   strcpy( last_uid, uid ); /* save uid of the last message for exit */
 	   if( PopUID[0] == 0 )
 	     break;  /* no previous UID set */
 	 }
@@ -258,6 +415,7 @@ void mutt_fetchPopMail (void)
 
   total = msgs - first_msg + 1; /* will be used for display later */
  
+  DM( "Status: total %d, messages %d, first_msg %d", total, msgs, first_msg );
 
   /*  Check for the total amount of bytes mail to be received */
   tot_bytes=0;
@@ -267,39 +425,47 @@ void mutt_fetchPopMail (void)
     write (s, buffer, strlen(buffer));
     if ( getLine (s, buffer, sizeof(buffer)) == -1) 
     {
+	    DM( "Abort while listing message %d", i );
       mx_fastclose_mailbox (&ctx);
       goto fail;
     }
     
     if (sscanf (buffer,"+OK %d %d",&i,&num_bytes) != 2) 
     {
+	    DM( "Error on list message %d encountered", i );
       tot_bytes=-1;
       break;
     }
     tot_bytes += num_bytes;
+
+    DM( "Message info: total %d, this %d", tot_bytes, num_bytes );
   }
 
 
   num_bytes=0;  
   for (i = first_msg ; i <= msgs ; i++)
   {
+	  DM( "Retrieving message %d", i );
+
     snprintf (buffer, sizeof(buffer), "retr %d\r\n", i);
     write (s, buffer, strlen (buffer));
 
 #ifdef BALSA_USE_THREADS
     sprintf( threadbuf, "Retrieving Message %d of %d", 
 	     i - first_msg + 1, total );
-    MSGMAILTHREAD( threadmsg, MSGMAILTHREAD_MSGINFO, threadbuf,0,0 );
+    prog_cb ( threadbuf, 0, 0);
 #endif
 
     if (getLine (s, buffer, sizeof (buffer)) == -1)
     {
+	    DM( "Abort while retrieving message %d", i );
       mx_fastclose_mailbox (&ctx);
       goto fail;
     }
 
     if (mutt_strncmp (buffer, "+OK", 3) != 0)
     {
+	    DM( "Error on retrieve message %d", i );
       mutt_remove_trailing_ws (buffer);
       mutt_error (buffer);
       break;
@@ -307,6 +473,7 @@ void mutt_fetchPopMail (void)
 
     if ((msg = mx_open_new_message (&ctx, NULL, M_ADD_FROM)) == NULL)
     {
+	    DM( "Error while creating new message %d", i );
       err = 1;
       break;
     }
@@ -317,6 +484,8 @@ void mutt_fetchPopMail (void)
       char *p;
       int chunk;
 
+      DM( "Message reading loop: %d" );
+
       if ((chunk = getLine (s, buffer, sizeof (buffer))) == -1)
       {
 	mutt_error ("Error reading message!");
@@ -326,13 +495,17 @@ void mutt_fetchPopMail (void)
       
 #ifdef BALSA_USE_THREADS
       sprintf( threadbuf,"Received %d bytes of %d",num_bytes,tot_bytes);
-      MSGMAILTHREAD(threadmsg, MSGMAILTHREAD_PROGRESS, threadbuf, num_bytes,tot_bytes); 
+      prog_cb ( threadbuf, num_bytes, tot_bytes );
 #endif
+
       /* check to see if we got a full line */
       if (buffer[chunk-2] == '\r' && buffer[chunk-1] == '\n')
       {
+	      DM( "Line completed" );
+
 	if (mutt_strcmp(".\r\n", buffer) == 0)
 	{
+		DM( "Line finished" );
 	  /* end of message */
 	  break;
 	}
@@ -355,7 +528,8 @@ void mutt_fetchPopMail (void)
       }
       else
 	p = buffer;
-      
+
+      DM( "Writing message to fp" );
       fwrite (p, 1, chunk, msg->fp);
     }
 
@@ -365,6 +539,8 @@ void mutt_fetchPopMail (void)
       err = 1;
     }
 
+    DM( "Message retrieved" );
+
     mx_close_message (&msg);
 
     if (err)
@@ -372,6 +548,8 @@ void mutt_fetchPopMail (void)
 
     if (option (OPTPOPDELETE))
     {
+	    DM( "Nuking message" );
+
       /* delete the message on the server */
       snprintf (buffer, sizeof(buffer), "dele %d\r\n", i);
       write (s, buffer, mutt_strlen (buffer));
@@ -381,6 +559,7 @@ void mutt_fetchPopMail (void)
       if (mutt_strncmp (buffer, "+OK", 3) != 0)
       {
 	err = 1;
+	DM( "Error on message delete %d", i );
         mutt_remove_trailing_ws (buffer);
 	mutt_error (buffer);
 	break;
@@ -392,13 +571,14 @@ void mutt_fetchPopMail (void)
 
   if (msg)
   {
+	  DM( "commit close message" );
     mx_commit_message (msg, &ctx);
     mx_close_message (&msg);
   }
-  mx_close_mailbox (&ctx);
 
   if (err)
   {
+	  DM( "Reset server" );
     /* make sure no messages get deleted */
     write (s, "rset\r\n", 6);
     getLine (s, buffer, sizeof (buffer)); /* snarf the response */
@@ -406,6 +586,10 @@ void mutt_fetchPopMail (void)
 
 finish:
 
+  DM( "mailbox close" );
+  mx_close_mailbox (&ctx, NULL);
+
+  DM( "Logout from server" );
   /* exit gracefully */
   write (s, "quit\r\n", 6);
   getLine (s, buffer, sizeof (buffer)); /* snarf the response */
@@ -416,7 +600,7 @@ finish:
   /* not reached */
 
 fail:
-
+  DM( "Abort failure death explosion" );
   mutt_error ("Server closed connection!");
   close (s);
 }
