@@ -61,6 +61,9 @@ int _mutt_traverse_thread (CONTEXT *ctx, HEADER *hdr, int flag);
 
 typedef const char * format_t (char *, size_t, char, const char *, const char *, const char *, const char *, unsigned long, format_flag);
 
+/* BALSA: Added this for use with mutt_fetchPopMail */
+typedef void (*muttProgressCallback)(char *msg, int prog, int tot);
+
 void mutt_FormatString (char *, size_t, const char *, format_t *, unsigned long, format_flag);
 void mutt_parse_content_type (char *, BODY *);
 void mutt_update_encoding (BODY *a);
@@ -79,6 +82,7 @@ ADDRESS *mutt_expand_aliases (ADDRESS *);
 ADDRESS *mutt_parse_adrlist (ADDRESS *, const char *);
 
 BODY *mutt_dup_body (BODY *);
+BODY *mutt_copy_body (BODY *, HEADER *hdr);
 BODY *mutt_make_file_attach (const char *);
 BODY *mutt_make_message_attach (CONTEXT *, HEADER *, int);
 BODY *mutt_remove_multipart (BODY *);
@@ -88,7 +92,7 @@ BODY *mutt_parse_multipart (FILE *, const char *, long, int);
 BODY *mutt_parse_messageRFC822 (FILE *, BODY *);
 BODY *mutt_read_mime_header (FILE *, int);
 
-ENVELOPE *mutt_read_rfc822_header (FILE *, HEADER *, short);
+ENVELOPE *mutt_read_rfc822_header (FILE *, HEADER *, short, short);
 HEADER *mutt_dup_header (HEADER *);
 
 ATTACHPTR **mutt_gen_attach_list (BODY *, int, ATTACHPTR **, short *, short *, int, int);
@@ -147,7 +151,9 @@ void mutt_exit (int);
 void mutt_expand_file_fmt (char *, size_t, const char *, const char *);
 void mutt_expand_fmt (char *, size_t, const char *, const char *);
 void mutt_expand_link (char *, const char *, const char *);
-void mutt_fetchPopMail (void);
+
+/* BALSA: Added the callback paramter */
+void mutt_fetchPopMail (muttProgressCallback prog_cb);
 void mutt_folder_hook (char *);
 void mutt_free_alias (ALIAS **);
 void mutt_free_body (BODY **);
@@ -178,7 +184,7 @@ void mutt_query_menu (char *, size_t);
 void mutt_safe_path (char *s, size_t l, ADDRESS *a);
 void mutt_sanitize_filename (char *, short);
 void mutt_save_path (char *s, size_t l, ADDRESS *a);
-void mutt_score_message (HEADER *);
+void mutt_score_message (CONTEXT *, HEADER *, int);
 void mutt_select_fcc (char *, size_t, HEADER *);
 void mutt_select_file (char *, size_t, int);
 void mutt_send_hook (HEADER *);
@@ -298,6 +304,8 @@ void *safe_malloc (size_t);
 void safe_realloc (void **, size_t);
 void safe_free (void **);
 
+const char *mutt_stristr (const char *, const char *);
+
 int safe_open (const char *, int);
 int safe_symlink (const char *, const char *);
 FILE *safe_fopen (const char *, const char *);
@@ -314,6 +322,10 @@ ADDRESS *alias_reverse_lookup (ADDRESS *);
    on some systems */
 #define SKIPWS(c) while (*(c) && isspace ((unsigned char) *(c))) c++;
 
+/* base64.c */
+void mutt_to_base64 (unsigned char*, const unsigned char*, int);
+int mutt_from_base64 (char*, const char*);
+
 #ifdef LOCALES_HACK
 #define IsPrint(c) (isprint((unsigned char)(c)) || \
 	((unsigned char)(c) >= 0xa0))
