@@ -1,5 +1,7 @@
+/* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
- * Copyright (C) 1998 Stuart Parmenter
+ * Copyright (C) 1997-2000 Stuart Parmenter and others,
+ *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,8 +22,15 @@
 #ifndef __BALSA_ICONS_H__
 #define __BALSA_ICONS_H__
 
-typedef enum
-  {
+#define BALSA_PIXMAP_MAIL_RPL_ALL 	"reply_to_all"
+#define BALSA_PIXMAP_MAIL_RPL_ALL_MENU	"reply_to_all_menu"
+
+#define BALSA_PIXMAP_NEXT_UNREAD "next_unread"
+#define BALSA_PIXMAP_NEXT_UNREAD_MENU "next_unread_menu"
+
+#define BALSA_PIXMAP_FLAGGED "flagged"
+
+typedef enum {
     BALSA_ICON_INBOX,
     BALSA_ICON_OUTBOX,
     BALSA_ICON_TRASH,
@@ -35,12 +44,15 @@ typedef enum
     BALSA_ICON_REPLIED,
     BALSA_ICON_FORWARDED,
 
-    BALSA_ICON_ENVELOPE
-  }
-BalsaIconName;
+    BALSA_ICON_ENVELOPE,
 
-void balsa_icons_init (void);
-GdkPixmap *balsa_icon_get_pixmap (BalsaIconName icon);
-GdkBitmap *balsa_icon_get_bitmap (BalsaIconName icon);
+    BALSA_ICON_ARROW,
+    BALSA_ICON_MULTIPART,
+} BalsaIconName;
 
+void balsa_icons_init(void);
+GdkPixmap *balsa_icon_get_pixmap(BalsaIconName icon);
+GdkBitmap *balsa_icon_get_bitmap(BalsaIconName icon);
+void register_balsa_pixmaps(void);
+void register_balsa_pixmap(gchar * name, gchar ** data, guint xsize, guint ysize);
 #endif
